#version 450
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
layout(early_fragment_tests) in;

struct lightingInput_t
{
    vec3 albedo;
    vec3 colorMask;
    vec3 specular;
    float smoothness;
    float maskSSS;
    float thickness;
    vec3 normal;
    vec3 normalTS;
    vec3 normalSSS;
    vec3 lightmap;
    vec3 emissive;
    float alpha;
    float ssdoDiffuseMul;
    vec3 view;
    vec3 position;
    vec4 texCoords;
    vec4 fragCoord;
    mat3 invTS;
    vec3 ambient_lighting;
    vec3 diffuse_lighting;
    vec3 specular_lighting;
    vec3 output_lighting;
    uint albedo_packed;
    uint specular_packed;
    uint diffuse_lighting_packed;
    uint specular_lighting_packed;
    uint normal_packed;
    uvec4 ticksDecals;
    uvec4 ticksProbes;
    uvec4 ticksLights;
};

struct clusternumlights_t
{
    int offset;
    int numItems;
};

struct light_t
{
    vec3 pos;
    uint lightParms;
    vec4 posShadow;
    vec4 falloffR;
    vec4 projS;
    vec4 projT;
    vec4 projQ;
    uvec4 scaleBias;
    vec4 boxMin;
    vec4 boxMax;
    vec4 areaPlane;
    uint lightParms2;
    uint colorPacked;
    float specMultiplier;
    float shadowBleedReduce;
};

struct shadowparms_t
{
    vec4 shadowLightS;
    vec4 shadowLightT;
    vec4 shadowLightR;
    vec4 shadowLightQ;
    vec4 shadowAtlasScaleBias;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 30, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 0, binding = 6, std140) uniform freqLow_fragmentUniforms_ubo
{
    vec4 globalvieworigin;
    vec4 vmtrfeedbackfloat;
    vec4 virtualtextureoptions;
    vec4 vmtrphysicalpagesaspectratio;
    vec4 resolutionscale;
    vec4 projectionmatrixz;
    vec4 clusteredlighting;
    vec4 envirradiancescale;
    vec4 pbrdebugparms;
    vec4 parallellightslicescale;
    vec4 shadowatlasresolution;
    vec4 highlightcolor;
} freqLow_fragmentUniforms;

layout(set = 0, binding = 5, std140) uniform freqHigh_fragmentUniforms_ubo
{
    vec4 usevirtualmapping;
    vec4 virtualmapping;
    vec4 physicalfilterparms;
    vec4 positiontoviewtexture;
    vec4 bloommaskscale;
    vec4 colorscale;
    vec4 color;
    vec4 bloomcolor;
    vec4 eyeirisdepth;
    vec4 eyeirisalbedo;
    vec4 eyeparms;
    vec4 bloodcolorlerp;
    vec4 bloodcolor;
    vec4 staticmodel;
} freqHigh_fragmentUniforms;

layout(set = 0, binding = 20, std430) restrict readonly buffer clusternumlights_b
{
    clusternumlights_t clusternumlights[];
} _2190;

layout(set = 0, binding = 21, std430) restrict readonly buffer clusterlightsid_b
{
    uint clusterlightsid[];
} _2371;

layout(set = 0, binding = 7, std140) uniform lightparms1_ubo
{
    vec4 lightparms1[4096];
} _2399;

layout(set = 0, binding = 8, std140) uniform lightparms2_ubo
{
    vec4 lightparms2[4096];
} _2438;

layout(set = 0, binding = 9, std140) uniform lightparms3_ubo
{
    vec4 lightparms3[4096];
} _2506;

layout(set = 0, binding = 10, std140) uniform shadowsparmsubo_ubo
{
    vec4 shadowsparmsubo[4095];
} _2836;

layout(set = 0, binding = 11) uniform SAMPLER_2D samp_vmtrpagetablemap;
layout(set = 0, binding = 12) uniform SAMPLER_2D samp_vmtrphysicalmappingsmap;
layout(set = 0, binding = 13) uniform sampler2DArray samp_vmtrphysicalpagesmaparray0;
layout(set = 0, binding = 14) uniform SAMPLER_2D samp_vmtrphysicalpagesmap3;
layout(set = 0, binding = 22, r32f) uniform writeonly image2D ib_feedbackbuffer;
layout(set = 0, binding = 15) uniform SAMPLER_2D samp_sparediffusemap;
layout(set = 0, binding = 16) uniform SAMPLER_2D samp_sparebumpmap;
layout(set = 0, binding = 17) uniform SAMPLER_2D samp_woundtexture;
layout(set = 0, binding = 18) uniform SAMPLER_2D samp_lightsatlasmap;
layout(set = 0, binding = 19) uniform sampler2DShadow samp_shadowatlasdepthcomparemap;

layout(location = 2) in vec4 vofi_TexCoord0;
layout(location = 6) in vec4 vofi_TexCoord4;
layout(location = 3) in vec4 vofi_TexCoord1;
layout(location = 4) in vec4 vofi_TexCoord2;
layout(location = 5) in vec4 vofi_TexCoord3;
layout(location = 9) in vec4 vofi_TexCoord7;
layout(location = 8) in vec4 vofi_TexCoord6;
layout(location = 10) in vec4 vofi_TexCoord8;
layout(location = 11) in vec4 vofi_TexCoord9;
layout(location = 12) in vec4 vofi_TexCoord10;
layout(location = 0) out vec4 out_FragColor0;
layout(location = 1) out vec4 out_FragColor1;
layout(location = 2) out vec4 out_FragColor2;

float _433;

float dot2(vec2 a, vec2 b)
{
    return dot(a, b);
}

vec4 tex2Dlod(SAMPLER_2D image, vec4 texcoord)
{
    return TEXTURE_LOD(image, texcoord.xy, texcoord.w);
}

vec4 tex2D(SAMPLER_2D image, vec2 texcoord)
{
    return SAMPLE_TEXTURE(image, texcoord);
}


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
vec4 tex2DARRAY(sampler2DArray image, vec3 texcoord, vec2 dx, vec2 dy)
{
    return textureGrad(image, texcoord, dx, dy);
}

vec4 tex2DArrayGrad(sampler2DArray image, float slice, vec2 texcoord, vec2 dx, vec2 dy)
{
    vec3 param = vec3(texcoord, slice);
    vec2 param_1 = dx;
    vec2 param_2 = dy;
    return tex2DARRAY(image, param, param_1, param_2);
}

vec4 texPhysArray(sampler2DArray image, float slice, vec2 texcoord, vec2 dx, vec2 dy)
{
    float param = slice;
    vec2 param_1 = texcoord;
    vec2 param_2 = dx;
    vec2 param_3 = dy;
    return tex2DArrayGrad(image, param, param_1, param_2, param_3);
}

vec4 tex2D(SAMPLER_2D image, vec2 texcoord, vec2 dx, vec2 dy)
{
    return TEXTURE_GRAD(image, texcoord, dx, dy);
}

vec4 texPhys(SAMPLER_2D image, vec2 texcoord, vec2 dx, vec2 dy)
{
    vec2 param = texcoord;
    vec2 param_1 = dx;
    vec2 param_2 = dy;
    return tex2D(image, param, param_1, param_2);
}

float SRGBlinear(float value)
{
    if (value <= 0.040449999272823333740234375)
    {
        return value / 12.9200000762939453125;
    }
    else
    {
        return pow((value / 1.05499994754791259765625) + 0.0521326996386051177978515625, 2.400000095367431640625);
    }
}

float DeGamma(float value)
{
    float param = value;
    return SRGBlinear(param);
}

float saturate(float v)
{
    return clamp(v, 0.0, 1.0);
}

vec2 screenPosToTexcoord(vec2 pos, vec4 bias_scale)
{
    return (pos * bias_scale.zw) + bias_scale.xy;
}

float asfloat(uint x)
{
    return uintBitsToFloat(x);
}

float sqr(float x)
{
    return x * x;
}

float SRGBlinearApprox(float value)
{
    return value * ((value * ((value * 0.305306017398834228515625) + 0.6821711063385009765625)) + 0.01252287812530994415283203125);
}

vec3 SRGBlinearApprox(vec3 sRGB)
{
    float param = sRGB.x;
    vec3 outLinear;
    outLinear.x = SRGBlinearApprox(param);
    float param_1 = sRGB.y;
    outLinear.y = SRGBlinearApprox(param_1);
    float param_2 = sRGB.z;
    outLinear.z = SRGBlinearApprox(param_2);
    return outLinear;
}

vec4 SRGBlinearApprox(vec4 sRGBA)
{
    vec3 param = sRGBA.xyz;
    vec4 outLinear = vec4(SRGBlinearApprox(param), 1.0);
    float param_1 = sRGBA.w;
    outLinear.w = SRGBlinearApprox(param_1);
    return outLinear;
}

vec4 tex2DDeGammaApprox(SAMPLER_2D image, vec2 texcoord)
{
    vec2 param = texcoord;
    vec4 color = tex2D(image, param);
    vec4 param_1 = color;
    color = SRGBlinearApprox(param_1);
    return color;
}

float GetLuma(vec3 c)
{
    return dot(c, vec3(0.2125999927520751953125, 0.715200006961822509765625, 0.072200000286102294921875));
}

vec3 saturate(vec3 v)
{
    return clamp(v, vec3(0.0), vec3(1.0));
}


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
vec4 tex2Dbias(SAMPLER_2D image, vec4 texcoord)
{
    return texture(image, vec3(texcoord.xy, VIEW), texcoord.w);
}

vec3 ReconstructNormal(vec3 normalTS, bool isFrontFacing)
{
    vec3 N = (normalTS * 2.0) - vec3(1.0);
    float param = (1.0 - (N.x * N.x)) - (N.y * N.y);
    N.z = sqrt(saturate(param));
    if (isFrontFacing == false)
    {
        N.z = -N.z;
    }
    return N;
}

vec3 TransformNormal(vec3 normal, mat3 mat)
{
    return normalize(vec3(dot(normal, mat[0]), dot(normal, mat[1]), dot(normal, mat[2])));
}

vec3 GetWorldSpaceNormal(vec3 normalTS, mat3 invTS, bool isFrontFacing)
{
    vec3 param = normalTS;
    bool param_1 = isFrontFacing;
    vec3 N = ReconstructNormal(param, param_1);
    vec3 param_2 = N;
    mat3 param_3 = invTS;
    return TransformNormal(param_2, param_3);
}

float GetLinearDepth(float ndcZ, vec4 projectionMatrixZ, float rcpFarZ, bool bFirstPersonArmsRescale)
{
    float linearZ = projectionMatrixZ.w / (ndcZ + projectionMatrixZ.z);
    if (bFirstPersonArmsRescale)
    {
        float _901;
        if (linearZ < 1.0)
        {
            _901 = 10.0;
        }
        else
        {
            _901 = 1.0;
        }
        linearZ *= _901;
    }
    return linearZ * rcpFarZ;
}

uint packR10G10B10(inout vec3 value)
{
    vec3 param = value;
    value = saturate(param);
    return ((uint(value.x * 1023.0) << uint(20)) | (uint(value.y * 1023.0) << uint(10))) | uint(value.z * 1023.0);
}

uint asuint(float x)
{
    return floatBitsToUint(x);
}

float ApproxLog2(float f)
{
    float param = f;
    return (float(asuint(param)) / 8388608.0) - 127.0;
}

float ApproxExp2(float f)
{
    uint param = uint((f + 127.0) * 8388608.0);
    return asfloat(param);
}

vec4 saturate(vec4 v)
{
    return clamp(v, vec4(0.0), vec4(1.0));
}

uint packR8G8B8A8(inout vec4 value)
{
    vec4 param = value;
    value = saturate(param);
    return (((uint(value.x * 255.0) << uint(24)) | (uint(value.y * 255.0) << uint(16))) | (uint(value.z * 255.0) << uint(8))) | uint(value.w * 255.0);
}

uint packRGBE(vec3 value)
{
    float param = max(max(value.x, value.y), value.z);
    float sharedExp = ceil(ApproxLog2(param));
    float param_1 = sharedExp;
    vec4 param_2 = vec4(value / vec3(ApproxExp2(param_1)), (sharedExp + 128.0) / 255.0);
    vec4 param_3 = saturate(param_2);
    uint _696 = packR8G8B8A8(param_3);
    return _696;
}

float fmin3(float f1, float f2, float f3)
{
    return min(f1, min(f2, f3));
}

float fmax3(float f1, float f2, float f3)
{
    return max(f1, max(f2, f3));
}

vec4 unpackR15G15B15A15(uvec2 value)
{
    return vec4(float((value.x >> uint(16)) & 65535u), float(value.x & 65535u), float((value.y >> uint(16)) & 65535u), float(value.y & 65535u)) * 3.0518509447574615478515625e-05;
}

uint getCubeMapFaceID(float x, float y, float z)
{
    bool _363 = abs(z) >= abs(x);
    bool _371;
    if (_363)
    {
        _371 = abs(z) >= abs(y);
    }
    else
    {
        _371 = _363;
    }
    if (_371)
    {
        int _374;
        if (z < 0.0)
        {
            _374 = 5;
        }
        else
        {
            _374 = 4;
        }
        return uint(_374);
    }
    if (abs(y) >= abs(x))
    {
        int _392;
        if (y < 0.0)
        {
            _392 = 3;
        }
        else
        {
            _392 = 2;
        }
        return uint(_392);
    }
    int _403;
    if (x < 0.0)
    {
        _403 = 1;
    }
    else
    {
        _403 = 0;
    }
    return uint(_403);
}


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
float shadow2Ddepth(sampler2DShadow image, vec3 texcoord)
{
    return textureLod(image, vec3(texcoord.xy, texcoord.z), 0.0);
}

vec4 unpackR8G8B8A8(uint value)
{
    return vec4(float((value >> uint(24)) & 255u), float((value >> uint(16)) & 255u), float((value >> uint(8)) & 255u), float(value & 255u)) / vec4(255.0);
}

vec3 unpackRGBE(uint value)
{
    uint param = value;
    vec4 rgbe = unpackR8G8B8A8(param);
    float param_1 = (rgbe.w * 255.0) - 128.0;
    return rgbe.xyz * ApproxExp2(param_1);
}

vec2 saturate(vec2 v)
{
    return clamp(v, vec2(0.0), vec2(1.0));
}

vec3 unpackR10G10B10(uint value)
{
    return vec3(float((value >> uint(20)) & 1023u), float((value >> uint(10)) & 1023u), float(value & 1023u)) / vec3(1023.0);
}

float ApproxPow(float fBase, float fPower)
{
    float param = fBase;
    uint param_1 = uint((fPower * float(asuint(param))) - (((fPower - 1.0) * 127.0) * 8388608.0));
    return asfloat(param_1);
}

vec3 fresnelSchlick(vec3 f0, float costheta)
{
    float param = 50.0 * dot(f0, vec3(0.33329999446868896484375));
    float baked_spec_occl = saturate(param);
    float param_1 = 1.0 - costheta;
    float param_2 = saturate(param_1);
    float param_3 = 5.0;
    vec3 param_4 = f0 + ((vec3(baked_spec_occl) - f0) * ApproxPow(param_2, param_3));
    return saturate(param_4);
}

vec3 specBRDF(vec3 N, vec3 V, vec3 L, vec3 f0, float smoothness)
{
    vec3 H = normalize(V + L);
    float m = 1.0 - (smoothness * 0.800000011920928955078125);
    m *= m;
    m *= m;
    float m2 = m * m;
    float param = dot(N, H);
    float NdotH = saturate(param);
    float spec = ((NdotH * NdotH) * (m2 - 1.0)) + 1.0;
    spec = m2 / ((spec * spec) + 9.9999999392252902907785028219223e-09);
    float param_1 = dot(N, V);
    float Gv = (saturate(param_1) * (1.0 - m)) + m;
    float param_2 = dot(N, L);
    float Gl = (saturate(param_2) * (1.0 - m)) + m;
    spec /= (((4.0 * Gv) * Gl) + 9.9999999392252902907785028219223e-09);
    vec3 param_3 = f0;
    float param_4 = dot(L, H);
    return fresnelSchlick(param_3, param_4) * spec;
}

vec2 OctWrap(vec2 v)
{
    float _984;
    if (v.x >= 0.0)
    {
        _984 = 1.0;
    }
    else
    {
        _984 = -1.0;
    }
    float _993;
    if (v.y >= 0.0)
    {
        _993 = 1.0;
    }
    else
    {
        _993 = -1.0;
    }
    return (vec2(1.0) - abs(v.yx)) * vec2(_984, _993);
}

vec2 NormalOctEncode(inout vec3 n, bool compress_range)
{
    n /= vec3((abs(n.x) + abs(n.y)) + abs(n.z));
    vec2 _1019;
    if (n.z >= 0.0)
    {
        _1019 = n.xy;
    }
    else
    {
        vec2 param = n.xy;
        _1019 = OctWrap(param);
    }
    n = vec3(_1019.x, _1019.y, n.z);
    if (compress_range)
    {
        vec2 _1043 = (n.xy * 0.5) + vec2(0.5);
        n = vec3(_1043.x, _1043.y, n.z);
    }
    return n.xy;
}

int asint(float x)
{
    return floatBitsToInt(x);
}

float asfloat(int x)
{
    return intBitsToFloat(x);
}

float sqrtIEEEIntApproximation(float inX, int inSqrtConst)
{
    float param = inX;
    int x = asint(param);
    x = inSqrtConst + (x >> 1);
    int param_1 = x;
    return asfloat(param_1);
}

float fastSqrtNR0(float inX)
{
    float param = inX;
    int param_1 = 532487669;
    return sqrtIEEEIntApproximation(param, param_1);
}

float SmoothnessEncode(float s)
{
    float param = abs(s);
    float s1 = fastSqrtNR0(param);
    float _1055;
    if (s > 0.0)
    {
        _1055 = s1;
    }
    else
    {
        _1055 = -s1;
    }
    return (_1055 * 0.5) + 0.5;
}

void main()
{
    lightingInput_t innerLayer = lightingInput_t(vec3(0.0), vec3(0.0), vec3(0.0), 0.0, 0.0, 0.0, vec3(0.0), vec3(0.0, 0.0, 1.0), vec3(0.0, 0.0, 1.0), vec3(0.0), vec3(0.0), 1.0, 1.0, vec3(0.0), vec3(0.0), vec4(0.0), vec4(0.0), mat3(vec3(1.0, 0.0, 0.0), vec3(0.0, 1.0, 0.0), vec3(0.0, 0.0, 1.0)), vec3(0.0), vec3(0.0), vec3(0.0), vec3(0.0), 0u, 0u, 0u, 0u, 0u, uvec4(0u), uvec4(0u), uvec4(0u));
    lightingInput_t inputs = lightingInput_t(vec3(0.0), vec3(0.0), vec3(0.0), 0.0, 0.0, 0.0, vec3(0.0), vec3(0.0, 0.0, 1.0), vec3(0.0, 0.0, 1.0), vec3(0.0), vec3(0.0), 1.0, 1.0, vec3(0.0), vec3(0.0), vec4(0.0), vec4(0.0), mat3(vec3(1.0, 0.0, 0.0), vec3(0.0, 1.0, 0.0), vec3(0.0, 0.0, 1.0)), vec3(0.0), vec3(0.0), vec3(0.0), vec3(0.0), 0u, 0u, 0u, 0u, 0u, uvec4(0u), uvec4(0u), uvec4(0u));
    inputs.texCoords = vofi_TexCoord0;
    inputs.fragCoord = gl_FragCoord;
    inputs.position = freqLow_fragmentUniforms.globalvieworigin.xyz + vofi_TexCoord4.xyz;
    inputs.view = normalize(-vofi_TexCoord4.xyz);
    inputs.invTS = mat3(vec3(vofi_TexCoord1.xyz), vec3(vofi_TexCoord2.xyz), vec3(vofi_TexCoord3.xyz));
    uint feedback = 0u;
    vec4 sampleLightmap = vec4(0.0);
    vec4 sampleColorMask = vec4(0.0);
    vec4 sampleDiffuse = vec4(0.0);
    vec4 sampleSpecular = vec4(0.0);
    vec4 sampleNormal = vec4(0.0);
    vec2 texCoords = inputs.texCoords.xy;
    vec2 texCoordsLOD = inputs.texCoords.xy;
    if (freqHigh_fragmentUniforms.usevirtualmapping.x > 0.0)
    {
        texCoords = (fract(inputs.texCoords.xy) * freqHigh_fragmentUniforms.virtualmapping.xy) + freqHigh_fragmentUniforms.virtualmapping.zw;
        texCoordsLOD = inputs.texCoords.xy * freqHigh_fragmentUniforms.virtualmapping.xy;
    }
    float widthInTexels = freqLow_fragmentUniforms.vmtrfeedbackfloat.z;
    float maxAnisoLog2 = freqHigh_fragmentUniforms.physicalfilterparms.z;
    vec2 texelCoords = texCoordsLOD * widthInTexels;
    vec2 dx = dFdx(texelCoords);
    vec2 dy = dFdy(texelCoords);
    vec2 param = dx;
    vec2 param_1 = dx;
    float px = dot2(param, param_1);
    vec2 param_2 = dy;
    vec2 param_3 = dy;
    float py = dot2(param_2, param_3);
    float maxLod = 0.5 * log2(max(px, py));
    float minLod = 0.5 * log2(min(px, py));
    float anisoLOD = maxLod - min(maxAnisoLog2, maxLod - minLod);
    float sampleLOD = anisoLOD;
    float widthInPages = freqLow_fragmentUniforms.vmtrfeedbackfloat.y;
    float feedbackBias = freqHigh_fragmentUniforms.physicalfilterparms.y;
    uvec2 feedbackPageUV = uvec2(texCoords * widthInPages);
    uint feedbackLOD = uint(min(freqHigh_fragmentUniforms.physicalfilterparms.w, max(0.0, anisoLOD) + feedbackBias));
    uint feedbackPageSource = uint(freqLow_fragmentUniforms.vmtrfeedbackfloat.x);
    feedback = feedbackPageUV.x & 255u;
    feedback |= ((feedbackPageUV.y & 255u) << uint(8));
    feedback |= ((feedbackPageUV.x >> uint(8)) << uint(16));
    feedback |= ((feedbackPageUV.y >> uint(8)) << uint(24));
    feedback |= (feedbackLOD << uint(19));
    feedback |= (feedbackPageSource << uint(27));
    sampleLOD += 0.0;
    vec3 physCoords;
    if (freqLow_fragmentUniforms.virtualtextureoptions.y != 0.0)
    {
        float trilinearFraction = fract(sampleLOD);
        float triStart = 0.375;
        float triEnd = 0.625;
        bool twoSamples;
        float sampleLOD1;
        float sampleLOD2;
        if (trilinearFraction <= triStart)
        {
            twoSamples = false;
            sampleLOD1 = sampleLOD - 0.5;
            sampleLOD2 = 0.0;
        }
        else
        {
            if (trilinearFraction >= triEnd)
            {
                twoSamples = false;
                sampleLOD1 = sampleLOD + 0.5;
                sampleLOD2 = 0.0;
            }
            else
            {
                twoSamples = true;
                sampleLOD1 = sampleLOD - 0.5;
                sampleLOD2 = sampleLOD + 0.5;
            }
        }
        vec4 virtCoordsLod = vec4(texCoords.x, texCoords.y, 0.0, sampleLOD1);
        vec4 param_4 = virtCoordsLod;
        vec2 physPage = tex2Dlod(samp_vmtrpagetablemap, param_4).xy;
        vec2 param_5 = physPage;
        vec4 xform = tex2D(samp_vmtrphysicalmappingsmap, param_5);
        physCoords.x = (texCoords.x * xform.x) + xform.z;
        physCoords.y = (texCoords.y * (xform.x * freqLow_fragmentUniforms.vmtrphysicalpagesaspectratio.x)) + xform.w;
        physCoords.z = xform.y;
        vec2 dx_1 = dFdx(texCoordsLOD) * physCoords.z;
        vec2 dy_1 = dFdy(texCoordsLOD) * physCoords.z;
        float param_6 = 0.0;
        vec2 param_7 = physCoords.xy;
        vec2 param_8 = dx_1;
        vec2 param_9 = dy_1;
        sampleDiffuse = texPhysArray(samp_vmtrphysicalpagesmaparray0, param_6, param_7, param_8, param_9);
        float param_10 = 1.0;
        vec2 param_11 = physCoords.xy;
        vec2 param_12 = dx_1;
        vec2 param_13 = dy_1;
        sampleSpecular = texPhysArray(samp_vmtrphysicalpagesmaparray0, param_10, param_11, param_12, param_13);
        float param_14 = 2.0;
        vec2 param_15 = physCoords.xy;
        vec2 param_16 = dx_1;
        vec2 param_17 = dy_1;
        sampleLightmap = texPhysArray(samp_vmtrphysicalpagesmaparray0, param_14, param_15, param_16, param_17);
        vec2 param_18 = physCoords.xy;
        vec2 param_19 = dx_1;
        vec2 param_20 = dy_1;
        sampleColorMask = texPhys(samp_vmtrphysicalpagesmap3, param_18, param_19, param_20);
        sampleNormal = vec4(0.0);
        float cover = sampleLightmap.z;
        float param_21 = sampleLightmap.w;
        float power = DeGamma(param_21);
        sampleNormal.y = sampleSpecular.w;
        sampleNormal.w = sampleDiffuse.w;
        sampleSpecular.w = cover;
        sampleLightmap.w = power;
        if (twoSamples)
        {
            vec4 virtCoordsLod_1 = vec4(texCoords.x, texCoords.y, 0.0, sampleLOD2);
            vec4 param_22 = virtCoordsLod_1;
            vec2 physPage_1 = tex2Dlod(samp_vmtrpagetablemap, param_22).xy;
            vec2 param_23 = physPage_1;
            vec4 xform_1 = tex2D(samp_vmtrphysicalmappingsmap, param_23);
            physCoords.x = (texCoords.x * xform_1.x) + xform_1.z;
            physCoords.y = (texCoords.y * (xform_1.x * freqLow_fragmentUniforms.vmtrphysicalpagesaspectratio.x)) + xform_1.w;
            physCoords.z = xform_1.y;
            vec2 dx_2 = dFdx(texCoordsLOD) * physCoords.z;
            vec2 dy_2 = dFdy(texCoordsLOD) * physCoords.z;
            float param_24 = 0.0;
            vec2 param_25 = physCoords.xy;
            vec2 param_26 = dx_2;
            vec2 param_27 = dy_2;
            vec4 sampleDiffuse2 = texPhysArray(samp_vmtrphysicalpagesmaparray0, param_24, param_25, param_26, param_27);
            float param_28 = 1.0;
            vec2 param_29 = physCoords.xy;
            vec2 param_30 = dx_2;
            vec2 param_31 = dy_2;
            vec4 sampleSpecular2 = texPhysArray(samp_vmtrphysicalpagesmaparray0, param_28, param_29, param_30, param_31);
            float param_32 = 2.0;
            vec2 param_33 = physCoords.xy;
            vec2 param_34 = dx_2;
            vec2 param_35 = dy_2;
            vec4 sampleLightmap2 = texPhysArray(samp_vmtrphysicalpagesmaparray0, param_32, param_33, param_34, param_35);
            vec2 param_36 = physCoords.xy;
            vec2 param_37 = dx_2;
            vec2 param_38 = dy_2;
            vec4 sampleColorMask2 = texPhys(samp_vmtrphysicalpagesmap3, param_36, param_37, param_38);
            vec4 samplePower2 = vec4(0.0);
            float cover_1 = sampleLightmap2.z;
            float param_39 = sampleLightmap2.w;
            float power_1 = DeGamma(param_39);
            samplePower2.y = sampleSpecular2.w;
            samplePower2.w = sampleDiffuse2.w;
            sampleSpecular2.w = cover_1;
            sampleLightmap2.w = power_1;
            float lerpFactor = 0.0;
            float triStart_1 = 0.375;
            float param_40 = (trilinearFraction - triStart_1) * 4.0;
            lerpFactor = saturate(param_40);
            sampleSpecular = mix(sampleSpecular, sampleSpecular2, vec4(lerpFactor));
            sampleDiffuse = mix(sampleDiffuse, sampleDiffuse2, vec4(lerpFactor));
            sampleNormal = mix(sampleNormal, samplePower2, vec4(lerpFactor));
            sampleLightmap = mix(sampleLightmap, sampleLightmap2, vec4(lerpFactor));
            sampleColorMask = mix(sampleColorMask, sampleColorMask2, vec4(lerpFactor));
        }
    }
    else
    {
        vec4 virtCoordsLod_2 = vec4(texCoords.x, texCoords.y, 0.0, sampleLOD - 0.5);
        vec4 param_41 = virtCoordsLod_2;
        vec2 physPage_2 = tex2Dlod(samp_vmtrpagetablemap, param_41).xy;
        vec2 param_42 = physPage_2;
        vec4 xform_2 = tex2D(samp_vmtrphysicalmappingsmap, param_42);
        physCoords.x = (texCoords.x * xform_2.x) + xform_2.z;
        physCoords.y = (texCoords.y * (xform_2.x * freqLow_fragmentUniforms.vmtrphysicalpagesaspectratio.x)) + xform_2.w;
        physCoords.z = xform_2.y;
        vec2 dx_3 = dFdx(texCoordsLOD) * physCoords.z;
        vec2 dy_3 = dFdy(texCoordsLOD) * physCoords.z;
        float param_43 = 0.0;
        vec2 param_44 = physCoords.xy;
        vec2 param_45 = dx_3;
        vec2 param_46 = dy_3;
        sampleDiffuse = texPhysArray(samp_vmtrphysicalpagesmaparray0, param_43, param_44, param_45, param_46);
        float param_47 = 1.0;
        vec2 param_48 = physCoords.xy;
        vec2 param_49 = dx_3;
        vec2 param_50 = dy_3;
        sampleSpecular = texPhysArray(samp_vmtrphysicalpagesmaparray0, param_47, param_48, param_49, param_50);
        float param_51 = 2.0;
        vec2 param_52 = physCoords.xy;
        vec2 param_53 = dx_3;
        vec2 param_54 = dy_3;
        sampleLightmap = texPhysArray(samp_vmtrphysicalpagesmaparray0, param_51, param_52, param_53, param_54);
        vec2 param_55 = physCoords.xy;
        vec2 param_56 = dx_3;
        vec2 param_57 = dy_3;
        sampleColorMask = texPhys(samp_vmtrphysicalpagesmap3, param_55, param_56, param_57);
        sampleNormal = vec4(0.0);
        float cover_2 = sampleLightmap.z;
        float param_58 = sampleLightmap.w;
        float power_2 = DeGamma(param_58);
        sampleNormal.y = sampleSpecular.w;
        sampleNormal.w = sampleDiffuse.w;
        sampleSpecular.w = cover_2;
        sampleLightmap.w = power_2;
    }
    vec2 TEXTURE_RESOLUTION = vec2(160.0, 120.0);
    vec2 FRAMEBUFFER_PIXEL_SIZE = TEXTURE_RESOLUTION * freqHigh_fragmentUniforms.positiontoviewtexture.zw;
    vec2 param_59 = gl_FragCoord.xy;
    vec4 param_60 = freqHigh_fragmentUniforms.positiontoviewtexture;
    vec2 tc = screenPosToTexcoord(param_59, param_60) * TEXTURE_RESOLUTION;
    float offsetMultiplier = floor(mod(tc.x + 0.25, 4.0));
    float xOffset = offsetMultiplier * FRAMEBUFFER_PIXEL_SIZE.x;
    bool _1720 = fract(tc.x - xOffset) < FRAMEBUFFER_PIXEL_SIZE.x;
    bool _1729;
    if (_1720)
    {
        _1729 = fract(tc.y) < FRAMEBUFFER_PIXEL_SIZE.y;
    }
    else
    {
        _1729 = _1720;
    }
    if (_1729)
    {
        uint param_61 = feedback;
        imageStore(ib_feedbackbuffer, ivec2(tc), vec4(asfloat(param_61), 0.0, 0.0, 0.0));
    }
    inputs.emissive = (vec3(((freqHigh_fragmentUniforms.bloommaskscale.x * sampleLightmap.x) * sampleLightmap.x) * freqHigh_fragmentUniforms.colorscale.x) * freqHigh_fragmentUniforms.color.xyz) * freqHigh_fragmentUniforms.bloomcolor.xyz;
    inputs.albedo = sampleDiffuse.xyz;
    inputs.specular = sampleSpecular.xyz;
    inputs.colorMask = sampleColorMask.xyz;
    inputs.lightmap = sampleLightmap.xyz;
    inputs.smoothness = sampleLightmap.w;
    inputs.normal = sampleNormal.wyz;
    inputs.ssdoDiffuseMul = 0.0;
    mat3 invTS = transpose(inputs.invTS);
    vec3 viewTS = normalize(vec3(dot(inputs.view, invTS[0]), dot(inputs.view, invTS[1]), dot(inputs.view, invTS[2])));
    int num_taps = 8;
    vec3 _step = vec3(-viewTS.xy, 1.0) / vec3(float(num_taps));
    vec3 curr = vec3(vofi_TexCoord0.xy, 0.0);
    vec2 prev = curr.xy;
    vec4 param_62 = vec4(curr.xy, 0.0, 0.0);
    float param_63 = tex2Dlod(samp_sparediffusemap, param_62).w;
    float h = sqr(param_63) * freqHigh_fragmentUniforms.eyeirisdepth.x;
    float prev_h = h;
    int tap = 0;
    for (;;)
    {
        bool _1855 = tap < num_taps;
        bool _1862;
        if (_1855)
        {
            _1862 = curr.z < h;
        }
        else
        {
            _1862 = _1855;
        }
        if (_1862)
        {
            prev = curr.xy;
            prev_h = h;
            curr += _step;
            vec4 param_64 = vec4(curr.xy, 0.0, 0.0);
            float param_65 = tex2Dlod(samp_sparediffusemap, param_64).w;
            h = sqr(param_65) * freqHigh_fragmentUniforms.eyeirisdepth.x;
            tap++;
            continue;
        }
        else
        {
            break;
        }
    }
    float z_after = h - curr.z;
    float z_before = (prev_h - curr.z) + _step.z;
    float param_66 = z_after / (z_after - z_before);
    float w = saturate(param_66);
    vec2 tc_parallax = (prev * w) + (curr.xy * (1.0 - w));
    vec2 tc_1 = vofi_TexCoord0.xy;
    vec2 param_67 = tc_parallax;
    vec4 albedo = tex2DDeGammaApprox(samp_sparediffusemap, param_67);
    innerLayer.albedo = albedo.xyz;
    inputs.albedo = albedo.xyz;
    vec3 param_68 = inputs.albedo;
    vec3 param_69 = (albedo.www * freqHigh_fragmentUniforms.eyeirisalbedo.w) * 8.0;
    inputs.albedo = mix(inputs.albedo, freqHigh_fragmentUniforms.eyeirisalbedo.xyz * GetLuma(param_68), saturate(param_69));
    inputs.specular = freqHigh_fragmentUniforms.eyeparms.xxx;
    inputs.smoothness = freqHigh_fragmentUniforms.eyeparms.y;
    vec2 param_70 = tc_1;
    inputs.normal = tex2D(samp_sparebumpmap, param_70).xyz;
    vec3 param_71 = innerLayer.albedo;
    innerLayer.specular = freqHigh_fragmentUniforms.eyeparms.zzz * GetLuma(param_71);
    innerLayer.smoothness = freqHigh_fragmentUniforms.eyeparms.w;
    vec4 param_72 = vec4(tc_parallax, 0.0, 0.0);
    innerLayer.normal = vec3(1.0) - tex2Dbias(samp_sparebumpmap, param_72).xyz;
    vec3 param_73 = innerLayer.normal;
    mat3 param_74 = inputs.invTS;
    bool param_75 = true;
    innerLayer.normal = GetWorldSpaceNormal(param_73, param_74, param_75);
    vec2 param_76 = vofi_TexCoord7.xy;
    vec4 woundMap = tex2D(samp_woundtexture, param_76);
    vec3 _2001 = woundMap.xyz - vec3(0.5);
    woundMap = vec4(_2001.x, _2001.y, _2001.z, woundMap.w);
    float param_77 = vofi_TexCoord6.x * 0.5;
    float param_78 = ((1.0 - saturate(param_77)) - (1.0 - woundMap.w)) * 3.0;
    float woundMask = saturate(param_78);
    vec3 diffuseBloodLerp = mix(inputs.albedo, freqHigh_fragmentUniforms.bloodcolorlerp.xyz, vec3(woundMask));
    inputs.albedo *= mix(vec3(1.0), freqHigh_fragmentUniforms.bloodcolor.xyz, vec3(woundMask));
    inputs.albedo = mix(inputs.albedo, diffuseBloodLerp, vec3(0.5));
    inputs.specular = mix(inputs.specular, vec3(0.0199999995529651641845703125), vec3(woundMask));
    inputs.smoothness = mix(inputs.smoothness, 0.64999997615814208984375, woundMask);
    vec2 _2074 = inputs.normal.xy + mix(woundMap.xy * vec2(woundMask), vec2(0.0), vec2(freqHigh_fragmentUniforms.bloodcolor.w));
    inputs.normal = vec3(_2074.x, _2074.y, inputs.normal.z);
    int _2079;
    if (freqHigh_fragmentUniforms.staticmodel.x > 0.0)
    {
        _2079 = 8;
    }
    else
    {
        _2079 = 0;
    }
    uint skipStaticModel = uint(_2079);
    float NUM_CLUSTERS_X = 16.0;
    float NUM_CLUSTERS_Y = 8.0;
    float NUM_CLUSTERS_Z = 24.0;
    vec2 param_79 = inputs.fragCoord.xy;
    vec4 param_80 = freqHigh_fragmentUniforms.positiontoviewtexture;
    vec2 _2105 = screenPosToTexcoord(param_79, param_80) * freqLow_fragmentUniforms.resolutionscale.zw;
    vec3 clusterCoordinate;
    clusterCoordinate = vec3(_2105.x, _2105.y, clusterCoordinate.z);
    clusterCoordinate.y = 1.0 - clusterCoordinate.y;
    vec2 _2118 = floor(clusterCoordinate.xy * vec2(NUM_CLUSTERS_X, NUM_CLUSTERS_Y));
    clusterCoordinate = vec3(_2118.x, _2118.y, clusterCoordinate.z);
    float param_81 = inputs.fragCoord.z;
    vec4 param_82 = freqLow_fragmentUniforms.projectionmatrixz;
    float param_83 = 1.0;
    bool param_84 = false;
    float curr_z = GetLinearDepth(param_81, param_82, param_83, param_84);
    float slice = log2(max(1.0, curr_z / freqLow_fragmentUniforms.clusteredlighting.z)) * freqLow_fragmentUniforms.clusteredlighting.w;
    clusterCoordinate.z = min(NUM_CLUSTERS_Z - 1.0, floor(slice));
    uint clusterOffset = uint((clusterCoordinate.x + (clusterCoordinate.y * NUM_CLUSTERS_X)) + ((clusterCoordinate.z * NUM_CLUSTERS_X) * NUM_CLUSTERS_Y));
    vec3 param_85 = inputs.albedo;
    uint _2167 = packR10G10B10(param_85);
    inputs.albedo_packed = _2167;
    vec3 param_86 = inputs.specular;
    uint _2173 = packR10G10B10(param_86);
    inputs.specular_packed = _2173;
    uint lightsMin = 0u;
    uint lightsMax = 0u;
    uint decalsMin = 0u;
    uint decalsMax = 0u;
    uint probesMin = 0u;
    uint probesMax = 0u;
    int MAX_LIGHTS_PER_CLUSTER = 256;
    clusternumlights_t cluster;
    cluster.offset = _2190.clusternumlights[clusterOffset].offset;
    cluster.numItems = _2190.clusternumlights[clusterOffset].numItems;
    int dataOffset = cluster.offset & (~(cluster.offset >> 31));
    int numItems = cluster.numItems & (~(cluster.numItems >> 31));
    lightsMin = uint((dataOffset >> 16) * MAX_LIGHTS_PER_CLUSTER);
    lightsMax = lightsMin + uint(numItems & 255);
    decalsMin = lightsMin;
    decalsMax = decalsMin + uint((numItems >> 8) & 255);
    probesMin = lightsMin;
    probesMax = probesMin + uint((numItems >> 16) & 255);
    bool isFrontFacing = true;
    vec3 param_87 = inputs.normal;
    bool param_88 = isFrontFacing;
    inputs.normalTS = ReconstructNormal(param_87, param_88);
    vec3 param_89 = inputs.normal;
    mat3 param_90 = inputs.invTS;
    bool param_91 = isFrontFacing;
    inputs.normal = GetWorldSpaceNormal(param_89, param_90, param_91);
    vec3 ambient = vec3(0.0);
    vec3 shc0 = vec3(vofi_TexCoord8.x, vofi_TexCoord8.y, vofi_TexCoord8.z);
    vec3 shc1 = vec3(vofi_TexCoord8.w, vofi_TexCoord9.x, vofi_TexCoord9.y);
    vec3 shc2 = vec3(vofi_TexCoord9.z, vofi_TexCoord9.w, vofi_TexCoord10.x);
    vec3 shc3 = vec3(vofi_TexCoord10.y, vofi_TexCoord10.z, vofi_TexCoord10.w);
    vec3 indirectDiffuse = ((shc0 - (shc1 * inputs.normal.y)) + (shc2 * inputs.normal.z)) - (shc3 * inputs.normal.x);
    float param_92 = freqLow_fragmentUniforms.envirradiancescale.x;
    ambient += max((indirectDiffuse * vec3(1.0)) * saturate(param_92), vec3(0.0));
    ambient += inputs.emissive;
    float _2326;
    if (dot(inputs.emissive, vec3(1.0)) > 0.0)
    {
        _2326 = -inputs.smoothness;
    }
    else
    {
        _2326 = inputs.smoothness;
    }
    inputs.smoothness = _2326;
    vec3 param_93 = ambient;
    ambient = mix(vec3(GetLuma(param_93)), ambient, freqLow_fragmentUniforms.pbrdebugparms.www);
    inputs.ambient_lighting = ambient;
    vec3 param_94 = ambient;
    inputs.diffuse_lighting_packed = packRGBE(param_94);
    uint lightIdx = lightsMin;
    light_t lightParms;
    shadowparms_t shadow_parms;
    shadowparms_t shadow_parms_1;
    shadowparms_t shadow_parms_2;
    int _3104;
    shadowparms_t shadow_parms_3;
    float _3594;
    float _3739;
    while (lightIdx < lightsMax)
    {
        uint temp = _2371.clusterlightsid[lightIdx];
        uint divergentLightId = temp & 4095u;
        uint light_id = divergentLightId;
        if (light_id >= divergentLightId)
        {
            lightIdx++;
        }
        int lightParms1Size = 4;
        int lightParms2Size = 4;
        int lightParms3Size = 3;
        lightParms.pos = _2399.lightparms1[(uint(lightParms1Size) * light_id) + 0u].xyz;
        float param_95 = _2399.lightparms1[(uint(lightParms1Size) * light_id) + 0u].w;
        lightParms.lightParms = asuint(param_95);
        lightParms.posShadow = _2399.lightparms1[(uint(lightParms1Size) * light_id) + 1u];
        lightParms.falloffR = _2399.lightparms1[(uint(lightParms1Size) * light_id) + 2u];
        lightParms.projS = _2438.lightparms2[(uint(lightParms2Size) * light_id) + 0u];
        lightParms.projT = _2438.lightparms2[(uint(lightParms2Size) * light_id) + 1u];
        lightParms.projQ = _2438.lightparms2[(uint(lightParms2Size) * light_id) + 2u];
        float param_96 = _2399.lightparms1[(uint(lightParms1Size) * light_id) + 3u].x;
        lightParms.scaleBias.x = asuint(param_96);
        float param_97 = _2399.lightparms1[(uint(lightParms1Size) * light_id) + 3u].y;
        lightParms.scaleBias.y = asuint(param_97);
        float param_98 = _2399.lightparms1[(uint(lightParms1Size) * light_id) + 3u].z;
        lightParms.scaleBias.z = asuint(param_98);
        float param_99 = _2399.lightparms1[(uint(lightParms1Size) * light_id) + 3u].w;
        lightParms.scaleBias.w = asuint(param_99);
        lightParms.boxMin = _2506.lightparms3[(uint(lightParms3Size) * light_id) + 0u];
        lightParms.boxMax = _2506.lightparms3[(uint(lightParms3Size) * light_id) + 1u];
        lightParms.areaPlane = _2506.lightparms3[(uint(lightParms3Size) * light_id) + 2u];
        float param_100 = _2438.lightparms2[(uint(lightParms2Size) * light_id) + 3u].x;
        lightParms.lightParms2 = asuint(param_100);
        float param_101 = _2438.lightparms2[(uint(lightParms2Size) * light_id) + 3u].y;
        lightParms.colorPacked = asuint(param_101);
        lightParms.specMultiplier = _2438.lightparms2[(uint(lightParms2Size) * light_id) + 3u].z;
        lightParms.shadowBleedReduce = _2438.lightparms2[(uint(lightParms2Size) * light_id) + 3u].w;
        float clip_min = 0.0039215688593685626983642578125;
        vec3 projTC = vec3((inputs.position.x * lightParms.projS.x) + ((inputs.position.y * lightParms.projS.y) + ((inputs.position.z * lightParms.projS.z) + lightParms.projS.w)), (inputs.position.x * lightParms.projT.x) + ((inputs.position.y * lightParms.projT.y) + ((inputs.position.z * lightParms.projT.z) + lightParms.projT.w)), (inputs.position.x * lightParms.projQ.x) + ((inputs.position.y * lightParms.projQ.y) + ((inputs.position.z * lightParms.projQ.z) + lightParms.projQ.w)));
        vec2 _2636 = projTC.xy / vec2(projTC.z);
        projTC = vec3(_2636.x, _2636.y, projTC.z);
        projTC.z = (inputs.position.x * lightParms.falloffR.x) + ((inputs.position.y * lightParms.falloffR.y) + ((inputs.position.z * lightParms.falloffR.z) + lightParms.falloffR.w));
        uint light_parms = lightParms.lightParms;
        uint light_flags = skipStaticModel;
        light_flags |= 64u;
        float param_102 = projTC.x;
        float param_103 = projTC.y;
        float param_104 = projTC.z;
        bool _2679 = fmin3(param_102, param_103, param_104) <= clip_min;
        bool _2696;
        if (!_2679)
        {
            float param_105 = projTC.x;
            float param_106 = projTC.y;
            float param_107 = projTC.z;
            _2696 = fmax3(param_105, param_106, param_107) >= (1.0 - clip_min);
        }
        else
        {
            _2696 = _2679;
        }
        bool _2704;
        if (!_2696)
        {
            _2704 = (light_parms & light_flags) != 0u;
        }
        else
        {
            _2704 = _2696;
        }
        if (_2704)
        {
            continue;
        }
        vec3 light_position = lightParms.pos;
        vec3 light_vector = light_position - inputs.position;
        float param_108 = dot(inputs.normal, light_vector);
        float NdotL = saturate(param_108);
        uvec2 param_109 = lightParms.scaleBias.xy;
        vec4 falloffScaleBias = unpackR15G15B15A15(param_109);
        uvec2 param_110 = lightParms.scaleBias.zw;
        vec4 projFilterScaleBias = unpackR15G15B15A15(param_110);
        vec4 param_111 = vec4((projTC.xy * projFilterScaleBias.xy) + projFilterScaleBias.zw, 0.0, 0.0);
        float projFilter = tex2Dlod(samp_lightsatlasmap, param_111).x;
        vec4 param_112 = vec4((vec2(projTC.z, 0.5) * falloffScaleBias.xy) + falloffScaleBias.zw, 0.0, 0.0);
        float falloff = tex2Dlod(samp_lightsatlasmap, param_112).x;
        float light_attenuation = (falloff * falloff) * (projFilter * projFilter);
        if (light_attenuation <= (clip_min / 256.0))
        {
            continue;
        }
        float shadow = 1.0;
        if ((light_parms & 4u) != 0u)
        {
            float normal_bias_scale = 0.5;
            vec3 pos = inputs.position + (inputs.normal * normal_bias_scale);
            uint idx = 0u;
            if ((light_parms & 3u) == 0u)
            {
                vec3 light_dir = pos - lightParms.posShadow.xyz;
                float param_113 = light_dir.x;
                float param_114 = light_dir.y;
                float param_115 = light_dir.z;
                idx = getCubeMapFaceID(param_113, param_114, param_115);
            }
            else
            {
                if ((light_parms & 3u) == 2u)
                {
                    float border = 1.0;
                    int shadowParmSize = 5;
                    shadow_parms.shadowLightS = _2836.shadowsparmsubo[(uint(shadowParmSize) * (light_parms >> uint(22))) + 0u];
                    shadow_parms.shadowLightT = _2836.shadowsparmsubo[(uint(shadowParmSize) * (light_parms >> uint(22))) + 1u];
                    shadow_parms.shadowLightR = _2836.shadowsparmsubo[(uint(shadowParmSize) * (light_parms >> uint(22))) + 2u];
                    shadow_parms.shadowLightQ = _2836.shadowsparmsubo[(uint(shadowParmSize) * (light_parms >> uint(22))) + 3u];
                    shadow_parms.shadowAtlasScaleBias = _2836.shadowsparmsubo[(uint(shadowParmSize) * (light_parms >> uint(22))) + 4u];
                    vec2 shadowTC = vec2(dot(pos, shadow_parms.shadowLightS.xyz) + shadow_parms.shadowLightS.w, dot(pos, shadow_parms.shadowLightT.xyz) + shadow_parms.shadowLightT.w);
                    shadowTC = abs((shadowTC * 2.0) - vec2(1.0));
                    bool _2908 = shadowTC.x >= border;
                    bool _2916;
                    if (!_2908)
                    {
                        _2916 = shadowTC.y >= border;
                    }
                    else
                    {
                        _2916 = _2908;
                    }
                    if (_2916)
                    {
                        idx = 1u;
                        int shadowParmSize_1 = 5;
                        shadow_parms_1.shadowLightS = _2836.shadowsparmsubo[(uint(shadowParmSize_1) * ((light_parms >> uint(22)) + 1u)) + 0u];
                        shadow_parms_1.shadowLightT = _2836.shadowsparmsubo[(uint(shadowParmSize_1) * ((light_parms >> uint(22)) + 1u)) + 1u];
                        shadow_parms_1.shadowLightR = _2836.shadowsparmsubo[(uint(shadowParmSize_1) * ((light_parms >> uint(22)) + 1u)) + 2u];
                        shadow_parms_1.shadowLightQ = _2836.shadowsparmsubo[(uint(shadowParmSize_1) * ((light_parms >> uint(22)) + 1u)) + 3u];
                        shadow_parms_1.shadowAtlasScaleBias = _2836.shadowsparmsubo[(uint(shadowParmSize_1) * ((light_parms >> uint(22)) + 1u)) + 4u];
                        shadowTC = vec2(dot(pos, shadow_parms_1.shadowLightS.xyz) + shadow_parms_1.shadowLightS.w, dot(pos, shadow_parms_1.shadowLightT.xyz) + shadow_parms_1.shadowLightT.w);
                        shadowTC = abs((shadowTC * 2.0) - vec2(1.0));
                        bool _2996 = shadowTC.x >= border;
                        bool _3004;
                        if (!_2996)
                        {
                            _3004 = shadowTC.y >= border;
                        }
                        else
                        {
                            _3004 = _2996;
                        }
                        if (_3004)
                        {
                            idx = 2u;
                            int shadowParmSize_2 = 5;
                            shadow_parms_2.shadowLightS = _2836.shadowsparmsubo[(uint(shadowParmSize_2) * ((light_parms >> uint(22)) + 2u)) + 0u];
                            shadow_parms_2.shadowLightT = _2836.shadowsparmsubo[(uint(shadowParmSize_2) * ((light_parms >> uint(22)) + 2u)) + 1u];
                            shadow_parms_2.shadowLightR = _2836.shadowsparmsubo[(uint(shadowParmSize_2) * ((light_parms >> uint(22)) + 2u)) + 2u];
                            shadow_parms_2.shadowLightQ = _2836.shadowsparmsubo[(uint(shadowParmSize_2) * ((light_parms >> uint(22)) + 2u)) + 3u];
                            shadow_parms_2.shadowAtlasScaleBias = _2836.shadowsparmsubo[(uint(shadowParmSize_2) * ((light_parms >> uint(22)) + 2u)) + 4u];
                            shadowTC = vec2(dot(pos, shadow_parms_2.shadowLightS.xyz) + shadow_parms_2.shadowLightS.w, dot(pos, shadow_parms_2.shadowLightT.xyz) + shadow_parms_2.shadowLightT.w);
                            shadowTC = abs((shadowTC * 2.0) - vec2(1.0));
                        }
                    }
                    float fraction = (1.0 - max(shadowTC.x, shadowTC.y)) * freqLow_fragmentUniforms.parallellightslicescale.w;
                    vec2 screenNoise = fract(gl_FragCoord.xy * 0.5);
                    screenNoise.x = fract((screenNoise.x * 0.5) + screenNoise.y);
                    if (screenNoise.x > fraction)
                    {
                        _3104 = 1;
                    }
                    else
                    {
                        _3104 = 0;
                    }
                    idx += uint(_3104);
                }
            }
            int shadowParmSize_3 = 5;
            shadow_parms_3.shadowLightS = _2836.shadowsparmsubo[(uint(shadowParmSize_3) * ((light_parms >> uint(22)) + idx)) + 0u];
            shadow_parms_3.shadowLightT = _2836.shadowsparmsubo[(uint(shadowParmSize_3) * ((light_parms >> uint(22)) + idx)) + 1u];
            shadow_parms_3.shadowLightR = _2836.shadowsparmsubo[(uint(shadowParmSize_3) * ((light_parms >> uint(22)) + idx)) + 2u];
            shadow_parms_3.shadowLightQ = _2836.shadowsparmsubo[(uint(shadowParmSize_3) * ((light_parms >> uint(22)) + idx)) + 3u];
            shadow_parms_3.shadowAtlasScaleBias = _2836.shadowsparmsubo[(uint(shadowParmSize_3) * ((light_parms >> uint(22)) + idx)) + 4u];
            vec4 shadowTC_1 = vec4((pos.x * shadow_parms_3.shadowLightS.x) + ((pos.y * shadow_parms_3.shadowLightS.y) + ((pos.z * shadow_parms_3.shadowLightS.z) + shadow_parms_3.shadowLightS.w)), (pos.x * shadow_parms_3.shadowLightT.x) + ((pos.y * shadow_parms_3.shadowLightT.y) + ((pos.z * shadow_parms_3.shadowLightT.z) + shadow_parms_3.shadowLightT.w)), (pos.x * shadow_parms_3.shadowLightR.x) + ((pos.y * shadow_parms_3.shadowLightR.y) + ((pos.z * shadow_parms_3.shadowLightR.z) + shadow_parms_3.shadowLightR.w)), (pos.x * shadow_parms_3.shadowLightQ.x) + ((pos.y * shadow_parms_3.shadowLightQ.y) + ((pos.z * shadow_parms_3.shadowLightQ.z) + shadow_parms_3.shadowLightQ.w)));
            vec3 _3260 = shadowTC_1.xyz / vec3(shadowTC_1.w);
            shadowTC_1 = vec4(_3260.x, _3260.y, _3260.z, shadowTC_1.w);
            bool _3265 = shadowTC_1.x > 0.0;
            bool _3271;
            if (_3265)
            {
                _3271 = shadowTC_1.x < 1.0;
            }
            else
            {
                _3271 = _3265;
            }
            bool _3277;
            if (_3271)
            {
                _3277 = shadowTC_1.y > 0.0;
            }
            else
            {
                _3277 = _3271;
            }
            bool _3283;
            if (_3277)
            {
                _3283 = shadowTC_1.y < 1.0;
            }
            else
            {
                _3283 = _3277;
            }
            if (_3283)
            {
                vec2 _3295 = (shadowTC_1.xy * shadow_parms_3.shadowAtlasScaleBias.xy) + shadow_parms_3.shadowAtlasScaleBias.zw;
                shadowTC_1 = vec4(_3295.x, _3295.y, shadowTC_1.z, shadowTC_1.w);
                if (false)
                {
                    vec3 param_116 = shadowTC_1.xyz;
                    shadow = shadow2Ddepth(samp_shadowatlasdepthcomparemap, param_116);
                }
                else
                {
                    vec2 uv = shadowTC_1.xy * freqLow_fragmentUniforms.shadowatlasresolution.xy;
                    vec2 shadowMapSizeInv = freqLow_fragmentUniforms.shadowatlasresolution.zw;
                    vec2 base_uv = vec2(floor(uv.x + 0.5), floor(uv.y + 0.5));
                    float s = (uv.x + 0.5) - base_uv.x;
                    float t = (uv.y + 0.5) - base_uv.y;
                    base_uv -= vec2(0.5);
                    base_uv *= shadowMapSizeInv;
                    float uw0 = 4.0 - (3.0 * s);
                    float uw1 = 7.0;
                    float uw2 = 1.0 + (3.0 * s);
                    float u0 = ((3.0 - (2.0 * s)) / uw0) - 2.0;
                    float u1 = (3.0 + s) / uw1;
                    float u2 = (s / uw2) + 2.0;
                    float vw0 = 4.0 - (3.0 * t);
                    float v0 = ((3.0 - (2.0 * t)) / vw0) - 2.0;
                    vec3 param_117 = vec3(base_uv + (vec2(u0, v0) * shadowMapSizeInv), shadowTC_1.z);
                    float s0 = (uw0 * vw0) * shadow2Ddepth(samp_shadowatlasdepthcomparemap, param_117);
                    vec3 param_118 = vec3(base_uv + (vec2(u1, v0) * shadowMapSizeInv), shadowTC_1.z);
                    float s1 = (uw1 * vw0) * shadow2Ddepth(samp_shadowatlasdepthcomparemap, param_118);
                    vec3 param_119 = vec3(base_uv + (vec2(u2, v0) * shadowMapSizeInv), shadowTC_1.z);
                    float s2 = (uw2 * vw0) * shadow2Ddepth(samp_shadowatlasdepthcomparemap, param_119);
                    shadow = (s0 + s1) + s2;
                    float vw1 = 7.0;
                    float v1 = (3.0 + t) / vw1;
                    vec3 param_120 = vec3(base_uv + (vec2(u0, v1) * shadowMapSizeInv), shadowTC_1.z);
                    float s0_1 = (uw0 * vw1) * shadow2Ddepth(samp_shadowatlasdepthcomparemap, param_120);
                    vec3 param_121 = vec3(base_uv + (vec2(u1, v1) * shadowMapSizeInv), shadowTC_1.z);
                    float s1_1 = (uw1 * vw1) * shadow2Ddepth(samp_shadowatlasdepthcomparemap, param_121);
                    vec3 param_122 = vec3(base_uv + (vec2(u2, v1) * shadowMapSizeInv), shadowTC_1.z);
                    float s2_1 = (uw2 * vw1) * shadow2Ddepth(samp_shadowatlasdepthcomparemap, param_122);
                    shadow += ((s0_1 + s1_1) + s2_1);
                    float vw2 = 1.0 + (3.0 * t);
                    float v2 = (t / vw2) + 2.0;
                    vec3 param_123 = vec3(base_uv + (vec2(u0, v2) * shadowMapSizeInv), shadowTC_1.z);
                    float s0_2 = (uw0 * vw2) * shadow2Ddepth(samp_shadowatlasdepthcomparemap, param_123);
                    vec3 param_124 = vec3(base_uv + (vec2(u1, v2) * shadowMapSizeInv), shadowTC_1.z);
                    float s1_2 = (uw1 * vw2) * shadow2Ddepth(samp_shadowatlasdepthcomparemap, param_124);
                    vec3 param_125 = vec3(base_uv + (vec2(u2, v2) * shadowMapSizeInv), shadowTC_1.z);
                    float s2_2 = (uw2 * vw2) * shadow2Ddepth(samp_shadowatlasdepthcomparemap, param_125);
                    shadow += ((s0_2 + s1_2) + s2_2);
                    shadow /= 144.0;
                }
            }
            else
            {
                if ((light_parms & 3u) == 2u)
                {
                    _3594 = 1.0;
                }
                else
                {
                    _3594 = 0.0;
                }
                shadow = _3594;
            }
            float shadow_fade = float((light_parms >> uint(16)) & 63u) / 63.0;
            float param_126 = ((shadow * shadow_fade) + 1.0) - shadow_fade;
            shadow = saturate(param_126);
        }
        uint param_127 = lightParms.colorPacked;
        vec3 light_color = (unpackRGBE(param_127) * shadow) * light_attenuation;
        float light_spec_multiplier = lightParms.specMultiplier;
        if ((light_parms & 48u) != 0u)
        {
            vec4 areaLightPlane = lightParms.areaPlane;
            vec4 areaLightPlaneU = lightParms.boxMin;
            vec4 areaLightPlaneV = lightParms.boxMax;
            float projToOrigin = dot(inputs.position, areaLightPlane.xyz) + areaLightPlane.w;
            if (projToOrigin < 0.0)
            {
                continue;
            }
            float falloffDistance = length(areaLightPlane.xyz);
            float param_128 = projToOrigin;
            float falloff_1 = saturate(param_128);
            light_color *= (falloff_1 * falloff_1);
            projToOrigin /= falloffDistance;
            areaLightPlane /= vec4(falloffDistance);
            vec3 mirrorDir = reflect(-inputs.view, inputs.normal);
            if (dot(mirrorDir, areaLightPlane.xyz) > 0.0)
            {
                mirrorDir = reflect(mirrorDir, areaLightPlane.xyz);
            }
            float projToPlane = dot(mirrorDir, areaLightPlane.xyz);
            float projLength = projToOrigin / projToPlane;
            vec3 hitPos = inputs.position - (mirrorDir * projLength);
            vec3 hitDir = hitPos - light_position;
            float u = dot(hitDir, areaLightPlaneU.xyz);
            float v = dot(hitDir, areaLightPlaneV.xyz);
            float lenCircle = length(vec2(u, v));
            float lenRect = max(abs(u), abs(v));
            if ((light_parms & 48u) == 32u)
            {
                _3739 = lenCircle;
            }
            else
            {
                _3739 = lenRect;
            }
            float len = _3739;
            if ((light_parms & 48u) == 48u)
            {
                vec2 param_129 = vec2(u, v);
                vec2 _3759 = saturate(param_129);
                projTC = vec3(_3759.x, _3759.y, projTC.z);
            }
            len = min(1.0, len) / len;
            u *= (len * areaLightPlaneU.w);
            v *= (len * areaLightPlaneV.w);
            hitDir = (areaLightPlaneU.xyz * u) + (areaLightPlaneV.xyz * v);
            light_position += hitDir;
        }
        vec3 light_color_final = light_color;
        vec3 param_130 = light_color_final;
        light_color_final = mix(vec3(GetLuma(param_130)), light_color_final, freqLow_fragmentUniforms.pbrdebugparms.www);
        vec3 light_vector_1 = normalize(light_position - inputs.position);
        float param_131 = dot(inputs.normal, light_vector_1);
        float NdotL_1 = saturate(param_131);
        uint param_132 = inputs.specular_packed;
        vec3 param_133 = inputs.normal;
        vec3 param_134 = inputs.view;
        vec3 param_135 = light_vector_1;
        vec3 param_136 = unpackR10G10B10(param_132);
        float param_137 = inputs.smoothness;
        vec3 spec = specBRDF(param_133, param_134, param_135, param_136, param_137);
        uint param_138 = inputs.specular_lighting_packed;
        vec3 param_139 = (((spec * light_spec_multiplier) * light_color_final) * NdotL_1) + unpackRGBE(param_138);
        inputs.specular_lighting_packed = packRGBE(param_139);
        float param_140 = 1.25;
        float param_141 = (dot(innerLayer.normal, light_vector_1) + 0.25) / sqr(param_140);
        float wrappedNdotL = saturate(param_141);
        uint param_142 = inputs.diffuse_lighting_packed;
        vec3 param_143 = (light_color_final * wrappedNdotL) + unpackRGBE(param_142);
        inputs.diffuse_lighting_packed = packRGBE(param_143);
        vec3 param_144 = innerLayer.normal;
        vec3 param_145 = inputs.view;
        vec3 param_146 = light_vector_1;
        vec3 param_147 = innerLayer.specular;
        float param_148 = innerLayer.smoothness;
        vec3 spec_1 = specBRDF(param_144, param_145, param_146, param_147, param_148);
        uint param_149 = inputs.specular_lighting_packed;
        vec3 param_150 = (((spec_1 * light_spec_multiplier) * light_color_final) * NdotL_1) + unpackRGBE(param_149);
        inputs.specular_lighting_packed = packRGBE(param_150);
    }
    uint param_151 = inputs.diffuse_lighting_packed;
    uint param_152 = inputs.albedo_packed;
    inputs.diffuse_lighting = unpackRGBE(param_151) * unpackR10G10B10(param_152);
    uint param_153 = inputs.specular_lighting_packed;
    inputs.specular_lighting = unpackRGBE(param_153);
    inputs.output_lighting = inputs.diffuse_lighting + inputs.specular_lighting;
    float param_154 = dot(inputs.normal, inputs.view);
    float NdotL_2 = saturate(param_154);
    uint param_155 = inputs.diffuse_lighting_packed;
    vec3 param_156 = unpackRGBE(param_155);
    uint param_157 = inputs.specular_packed;
    vec3 param_158 = inputs.normal;
    vec3 param_159 = inputs.view;
    vec3 param_160 = inputs.view;
    vec3 param_161 = unpackR10G10B10(param_157);
    float param_162 = inputs.smoothness;
    inputs.specular_lighting += ((saturate(param_156) * NdotL_2) * specBRDF(param_158, param_159, param_160, param_161, param_162));
    vec3 _3969 = inputs.diffuse_lighting + inputs.specular_lighting;
    out_FragColor0 = vec4(_3969.x, _3969.y, _3969.z, out_FragColor0.w);
    vec3 param_163 = inputs.normal;
    bool param_164 = false;
    vec2 _3977 = NormalOctEncode(param_163, param_164);
    out_FragColor1 = vec4(_3977.x, _3977.y, out_FragColor1.z, out_FragColor1.w);
    float param_165 = inputs.specular.x;
    float param_166 = inputs.specular.y;
    float param_167 = inputs.specular.z;
    float param_168 = inputs.smoothness;
    out_FragColor2 = vec4(fastSqrtNR0(param_165), fastSqrtNR0(param_166), fastSqrtNR0(param_167), SmoothnessEncode(param_168));
    vec3 _4003 = out_FragColor0.xyz + freqLow_fragmentUniforms.highlightcolor.xyz;
    out_FragColor0 = vec4(_4003.x, _4003.y, _4003.z, out_FragColor0.w);
}

